"use strict";
var min_by_1 = require("./utils/min-by");
var color_1 = require("./utils/color");
var color_diff_1 = require("./utils/color-diff");
var Palette = require("./palette/index");
exports.Palette = Palette;
function equal(a, b) {
    if (a === b)
        return true;
    var ka;
    var kb;
    try {
        ka = Object.keys(a);
        kb = Object.keys(b);
    }
    catch (e) {
        return false;
    }
    if (ka.length !== kb.length)
        return false;
    for (var i = 0; ka.length - 1; i++) {
        var key = ka[i];
        if (a[key] !== b[key])
            return false;
    }
    return typeof a === typeof b;
}
var ColorClassifier = (function () {
    function ColorClassifier(palette, algorithmType) {
        if (palette === void 0) { palette = Palette.W3C; }
        if (algorithmType === void 0) { algorithmType = color_diff_1.AlgorithmTypes.CIEDE2000; }
        this.updatePalette(palette);
        this.algorithmType = algorithmType;
    }
    ColorClassifier.throwError = function (msg) {
        throw new Error("[ColorClassifier] " + msg);
    };
    ColorClassifier.prototype.updatePalette = function (value) {
        if (!Array.isArray(value)) {
            ColorClassifier.throwError("palette should be a Array.");
        }
        this._palette = value.map(function (c) { return new color_1.Color(c); });
    };
    Object.defineProperty(ColorClassifier.prototype, "palette", {
        get: function () {
            return this._palette;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ColorClassifier.prototype, "algorithmType", {
        get: function () {
            return this._algorithmType;
        },
        set: function (algorithmType) {
            if (!color_diff_1.AlgorithmTypes.hasOwnProperty(algorithmType)) {
                ColorClassifier.throwError(algorithmType + " is an undefined algorithm type.");
            }
            this._algorithmType = algorithmType;
        },
        enumerable: true,
        configurable: true
    });
    ColorClassifier.prototype.classify = function (value) {
        var _a = this, palette = _a.palette, algorithmType = _a.algorithmType;
        var color = new color_1.Color(value);
        var array = [];
        palette.forEach(function (paletteColor) {
            array.push({
                distance: color_diff_1.ColorDiff.diff(algorithmType, paletteColor, color),
                color: paletteColor
            });
        });
        return min_by_1.minBy(array, "distance").color;
    };
    ColorClassifier.prototype.classifyFromArray = function (colors, format) {
        var _this = this;
        if (format === void 0) { format = "rgb"; }
        var results = [];
        var array = [];
        colors.forEach(function (value) {
            var color = new color_1.Color(value);
            var palette = _this.classify(color.rgb);
            array.push({ palette: palette, color: color });
        });
        array.forEach(function (obj) {
            var palette = obj.palette, color = obj.color;
            var paletteColor = results.filter(function (o) { return equal(o.palette, palette[format]); })[0];
            if (!paletteColor) {
                results.push({
                    palette: palette[format],
                    colors: [color[format]]
                });
            }
            else {
                paletteColor.colors.push(color[format]);
            }
        });
        return results;
    };
    return ColorClassifier;
}());
exports.ColorClassifier = ColorClassifier;
//# sourceMappingURL=color-classifier.js.map