"use strict";
var color_1 = require("./color");
var math_1 = require("./math");
var abs = Math.abs, atan2 = Math.atan2, cos = Math.cos, pow = Math.pow, PI = Math.PI, min = Math.min, exp = Math.exp, sqrt = Math.sqrt, sin = Math.sin;
exports.AlgorithmTypes = {
    CIEDE2000: "CIEDE2000",
    HSV: "HSV",
    RGB: "RGB"
};
var diffMethodMap = (_a = {},
    _a[exports.AlgorithmTypes.CIEDE2000] = "ciede2000",
    _a[exports.AlgorithmTypes.HSV] = "hsv",
    _a[exports.AlgorithmTypes.RGB] = "rgb",
    _a
);
var kl = 1;
var kc = 1;
var kh = 1;
var pow7_25 = math_1.pow7(25);
var ColorDiff = (function () {
    function ColorDiff() {
    }
    ColorDiff.diff = function (algorithmType, color1, color2) {
        var method = diffMethodMap[algorithmType];
        return ColorDiff[method](color1, color2);
    };
    ColorDiff.ciede2000 = function (color1, color2) {
        var a = color_1.Color.rgbToLab(color1.rgb);
        var b = color_1.Color.rgbToLab(color2.rgb);
        return ColorDiff._ciede2000(a.l, a.a, a.b, b.l, b.a, b.b);
    };
    ColorDiff._ciede2000 = function (l1, a1, b1, l2, a2, b2) {
        var c1 = math_1.hypot(a1, b1);
        var c2 = math_1.hypot(a2, b2);
        var ac1c2 = (c1 + c2) / 2;
        var g = 0.5 * (1 - sqrt(math_1.pow7(ac1c2) / (math_1.pow7(ac1c2) + pow7_25)));
        var a1p = (1 + g) * a1;
        var a2p = (1 + g) * a2;
        var c1p = sqrt(math_1.pow2(a1p) + math_1.pow2(b1));
        var c2p = sqrt(math_1.pow2(a2p) + math_1.pow2(b2));
        var h1pd = math_1.toDegrees(atan2(b1, a1p));
        var h1p = b1 === 0 && a1p === 0 ? 0 : h1pd > 0 ? h1pd : h1pd + 360;
        var h2pd = math_1.toDegrees(atan2(b2, a2p));
        var h2p = b2 === 0 && a2p === 0 ? 0 : h2pd > 0 ? h2pd : h2pd + 360;
        var dlp = l2 - l1;
        var dcp = c2p - c1p;
        var dhp = 2 * sqrt(c1p * c2p) * sin(math_1.toRadians(c1 * c2 === 0 ? 0 : abs(h2p - h1p) <= 180 ? h2p - h1p : (h2p - h1p) > 180 ? (h2p - h1p) - 360 : (h2p - h1p) + 360) / 2);
        var al = (l1 + l2) / 2;
        var acp = (c1p + c2p) / 2;
        var ahp;
        if (c1 * c2 === 0) {
            ahp = h1p + h2p;
        }
        else if (abs(h1p - h2p) <= 180) {
            ahp = (h1p + h2p) / 2;
        }
        else if (abs(h1p - h2p) > 180 && h1p + h2p < 360) {
            ahp = (h1p + h2p + 360) / 2;
        }
        else {
            ahp = (h1p + h2p - 360) / 2;
        }
        var t = 1 - 0.17 * cos(math_1.toRadians(ahp - 30)) + 0.24 * cos(math_1.toRadians(2 * ahp)) + 0.32 * cos(math_1.toRadians(3 * ahp + 6)) - 0.20 * cos(math_1.toRadians(4 * ahp - 63));
        var dro = 30 * exp(-(math_1.pow2((ahp - 275) / 25)));
        var rc = sqrt((math_1.pow7(acp)) / (math_1.pow7(acp) + pow7_25));
        var sl = 1 + ((0.015 * math_1.pow2(al - 50)) / sqrt(20 + math_1.pow2(al - 50)));
        var sc = 1 + 0.045 * acp;
        var sh = 1 + 0.015 * acp * t;
        var rt = -2 * rc * sin(math_1.toRadians(2 * dro));
        return sqrt(math_1.pow2(dlp / (sl * kl)) + math_1.pow2(dcp / (sc * kc)) + math_1.pow2(dhp / (sh * kh)) + rt * (dcp / (sc * kc)) * (dhp / (sh * kh)));
    };
    ColorDiff.hsv = function (color1, color2) {
        var a = color1.hsv;
        var b = color2.hsv;
        var h = 0;
        if (a.hue > b.hue) {
            h = min(a.hue - b.hue, b.hue - a.hue + 360);
        }
        else {
            h = min(b.hue - a.hue, a.hue - b.hue + 360);
        }
        return sqrt(math_1.pow2(h) + math_1.pow2(a.saturation - b.saturation) + math_1.pow2(a.value - b.value));
    };
    ColorDiff.rgb = function (color1, color2) {
        var a = color1.rgb;
        var b = color2.rgb;
        return sqrt(math_1.pow2(a.red - b.red) + math_1.pow2(a.green - b.green) + math_1.pow2(a.blue - b.blue));
    };
    return ColorDiff;
}());
exports.ColorDiff = ColorDiff;
var _a;
//# sourceMappingURL=color-diff.js.map